package org.dhaverdLogs;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.Period;
import java.time.format.DateTimeFormatter;
import java.util.List;

public class DhaverdLogs {
    public static final String osSeparator = System.getProperty("file.separator");
    public static final String indent = " ".repeat(22);
    public static String fileName = "log.log";
    public static String logDir = "";

    public static void logCleaner(boolean deleteBySevenDays) throws IOException {
        File dir = new File(logDir);
        if (!dir.isDirectory()){
            return;
        }
        File[] fileList = dir.listFiles();
        assert fileList != null;
        for (File file : fileList){
            String fileName = file.getName();
            if (!fileName.contains("gitignore")){
                if (deleteBySevenDays){
                    String[] params = fileName.split("_");
                    String date = params[0];
                    date = date.replace('.', '-');
                    String time = params[1];
                    time = time.replace('-', ':');
                    String dateTime = date + "T" + time;
                    LocalDateTime now = LocalDateTime.now();
                    LocalDateTime logDateTime = LocalDateTime.parse(dateTime);
                    Duration duration = Duration.between(logDateTime, now);
                    long diff = Math.abs(duration.toDays());
                    if (diff > 7){
                        setLog("Log cleaner", "File " + file.getName() + " is deleted: " + file.delete(), true, false);
                    }
                } else {
                    setLog("Log cleaner", "File " + file.getName() + " is deleted: " + file.delete(), true, false);
                }

            }
        }
    }


    public static void setLogName(String name){
        String time = getCurrentTime();
        time = time.replace("[", "");
        time = time.replace("]", "");
        time = time.replace("-", ".");
        time = time.replace(":", "-");
        time = time.replace(" ", "_");
        fileName = time + name + ".log";
    }

    public static void setLogDir(String dir){
        logDir = dir;
    }

    public static File getLogFile(){
        return new File(logDir + osSeparator + fileName);
    }

    public static void setLog(String context, String text, boolean addTime, boolean addIndent) throws IOException {
        File log = getLogFile();
        String finalText = "";
        if (addIndent){
            finalText += indent;
        }
        if (addTime){
            finalText += getCurrentTime();
        }
        finalText += "[" + context + "] ";
        finalText += text;
        finalText = finalText.replace("\n", "\n" + indent);
        FileWriter writer = new FileWriter(log, true);
        writer.write(finalText + "\r\n");
        System.out.println(finalText);
        writer.close();
    }

    public static String getCurrentTime(){
        String time;
        DateTimeFormatter dtf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime now = LocalDateTime.now();
        time = "[" + dtf.format(now) + "] ";
        return time;
    }

    public static void clearFile(File file) throws IOException {
        FileWriter writer = new FileWriter(file, false);
        writer.write("");
        writer.close();
    }

    public static boolean deleteFile(File file){
        return file.delete();
    }

    public static boolean exeptionActions(String logContext, Exception ex){
        try{
            setLog(logContext, "Ошибка!", false, false);
            setLog(logContext, ex.getLocalizedMessage() + "\n" + ex.getMessage(), true, false);
            for (StackTraceElement trace : ex.getStackTrace()){
                setLog(logContext, trace.toString(), false, true);
            }
        } catch (RuntimeException | IOException exc){
            exc.printStackTrace();
        }
        return true;
    }
}